/*! For license information please see 28.e233ef11.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[28],{4044:function(e,t,r){"use strict";r.r(t),r.d(t,"default",(function(){return i}));r(0);var s=r(347),a=r(5);var i=function(e){var t=e.element,r=e.width;return Object(a.jsx)("div",{className:"stProgress",children:Object(a.jsx)(s.b,{value:t.value,width:r})})}}}]);
//# sourceMappingURL=28.e233ef11.chunk.js.map