/*! For license information please see 24.1ef414d0.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[24],{4036:function(t,e,r){"use strict";r.r(e),r.d(e,"default",(function(){return a}));var i=r(0),n=r(4009),o=r(337),c=r(5);var a=Object(o.a)((function(t){var e=t.width,r=t.element,o=t.index,a=t.height,s="bokeh-chart-".concat(o),u=Object(i.useCallback)((function(){return JSON.parse(r.figure)}),[r]),h=Object(i.useCallback)((function(t){var i=document.getElementById(s),o=t&&t.doc&&t.doc.roots&&t.doc.roots.references?t.doc.roots.references.find((function(t){return"Plot"===t.type})):void 0;if(o){var c=function(t){var i=t.attributes.plot_width,n=t.attributes.plot_height;return a?(i=e,n=a):r.useContainerWidth&&(i=e),{chartWidth:i,chartHeight:n}}(o),u=c.chartWidth,h=c.chartHeight;u>0&&(o.attributes.plot_width=u),h>0&&(o.attributes.plot_height=h)}null!==i&&(!function(t){for(;t.lastChild;)t.lastChild.remove()}(i),n.a.embed_item(t,s))}),[e,a,r,o]);return Object(i.useEffect)((function(){h(u())}),[e,a,r,o,u,h]),Object(c.jsx)("div",{id:s,className:"stBokehChart"})}))}}]);
//# sourceMappingURL=24.1ef414d0.chunk.js.map