/*! For license information please see 20.ba137896.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[20],{3901:function(t,i){},4030:function(t,i,e){"use strict";e.r(i),e.d(i,"default",(function(){return m}));e(0);var n,r=e(3877),a=e.n(r),o=e(337),c=e(68),l=e(8),s=e.n(l),d=s()("div",{target:"etr89bj0"})((function(t){return{display:"flex",flexDirection:"row",flexWrap:"wrap",rowGap:t.theme.spacing.lg}}),""),u=s()("div",{target:"etr89bj1"})((function(t){t.theme;return{display:"flex",flexDirection:"column",alignItems:"stretch",width:"auto",flexGrow:0}}),""),h=s()("div",{target:"etr89bj2"})((function(t){var i=t.theme;return{fontFamily:i.genericFonts.bodyFont,fontSize:i.fontSizes.smDefault,color:i.colors.fadedText60,textAlign:"center",marginTop:i.spacing.xs}}),""),f=e(5);!function(t){t[t.OriginalWidth=-1]="OriginalWidth",t[t.ColumnWidth=-2]="ColumnWidth",t[t.AutoWidth=-3]="AutoWidth"}(n||(n={}));var m=Object(o.a)((function(t){var i,e=t.width,r=t.isFullScreen,o=t.element,l=t.height,s=o.width;if(s===n.OriginalWidth||s===n.AutoWidth)i=void 0;else if(s===n.ColumnWidth)i=e;else{if(!(s>0))throw Error("Invalid image width: ".concat(s));i=s}var m={};return l&&r?(m.maxHeight=l,m["object-fit"]="contain"):(m.width=i,s===n.AutoWidth&&(m.maxWidth="100%")),Object(f.jsx)(d,{style:{width:e},children:o.imgs.map((function(t,i){var e=t;return Object(f.jsxs)(u,{"data-testid":"stImage",children:[e.markup?a()(Object(c.e)(e.markup)):Object(f.jsx)("img",{style:m,src:Object(c.b)(e.url),alt:i.toString()}),e.caption&&Object(f.jsx)(h,{"data-testid":"caption",children:" ".concat(e.caption," ")})]},i)}))})}))}}]);
//# sourceMappingURL=20.ba137896.chunk.js.map