/*! For license information please see 15.6e16b12a.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[15],{3344:function(t,e){},3345:function(t,e){},3876:function(t,e){},4029:function(t,e,i){"use strict";i.r(e),i.d(e,"default",(function(){return d}));var n=i(0),c=i(3856),r=i(3874),h=i(14),a=i(337),o=i(8),s=i.n(o)()("div",{target:"e1p558ko0"})((function(t){return{"& *":{fontFamily:t.theme.genericFonts.bodyFont,fontSize:"9.6px"},"& svg":{maxWidth:"100%"}}}),""),u=i(5);r.graphviz;var d=Object(a.a)((function(t){var e=t.width,i=t.element,r=t.index,a=t.height,o="graphviz-chart-".concat(r),d=0,g=0,f=function(){var t=g,n=d;return a?(t=e,n=a):i.useContainerWidth&&(t=e),{chartWidth:t,chartHeight:n}},v=function(){try{var t=Object(c.select)("#".concat(o)).graphviz().zoom(!1).fit(!0).scale(1).renderDot(i.spec).on("end",(function(){var t=Object(c.select)("#".concat(o," > svg")).node();t&&(d=t.getBBox().height,g=t.getBBox().width)})),e=f(),n=e.chartHeight,r=e.chartWidth;n>0&&t.height(n),r>0&&t.width(r)}catch(a){Object(h.b)(a)}};Object(n.useEffect)((function(){v()}));var p=f(),b=p.chartWidth?p.chartWidth:e,l=p.chartHeight?p.chartHeight:a;return Object(u.jsx)(s,{className:"graphviz stGraphVizChart",id:o,style:{width:b,height:l}})}))}}]);
//# sourceMappingURL=15.6e16b12a.chunk.js.map