@font-face{font-family:"IBM Plex Mono";font-weight:400;font-style:regular;src:url(../../static/media/IBMPlexMono-Regular.bee53c8c.ttf) format("truetype")}@font-face{font-family:"IBM Plex Sans";font-weight:300;font-style:regular;src:url(../../static/media/IBMPlexSans-Light.0a619f93.ttf) format("truetype")}@font-face{font-family:"IBM Plex Sans";font-weight:400;font-style:regular;src:url(../../static/media/IBMPlexSans-Regular.71a4981f.ttf) format("truetype")}@font-face{font-family:"IBM Plex Sans";font-weight:500;font-style:regular;src:url(../../static/media/IBMPlexSans-Medium.c401b936.ttf) format("truetype")}@font-face{font-family:"IBM Plex Sans";font-weight:700;font-style:regular;src:url(../../static/media/IBMPlexSans-Bold.5a0eb98b.ttf) format("truetype")}@font-face{font-family:"IBM Plex Sans";font-weight:300;font-style:italic;src:url(../../static/media/IBMPlexSans-LightItalic.3a1f5a16.ttf) format("truetype")}@font-face{font-family:"IBM Plex Sans";font-weight:400;font-style:italic;src:url(../../static/media/IBMPlexSans-Italic.5c9d4db4.ttf) format("truetype")}@font-face{font-family:"IBM Plex Sans";font-weight:500;font-style:italic;src:url(../../static/media/IBMPlexSans-MediumItalic.3046879d.ttf) format("truetype")}@font-face{font-family:"IBM Plex Sans";font-weight:700;font-style:italic;src:url(../../static/media/IBMPlexSans-BoldItalic.d5548132.ttf) format("truetype")}@font-face{font-family:"IBM Plex Serif";font-weight:300;font-style:regular;src:url(../../static/media/IBMPlexSerif-Light.2e84bbf0.ttf) format("truetype")}@font-face{font-family:"IBM Plex Serif";font-weight:400;font-style:regular;src:url(../../static/media/IBMPlexSerif-Regular.9e4f976f.ttf) format("truetype")}@font-face{font-family:"IBM Plex Serif";font-weight:500;font-style:regular;src:url(../../static/media/IBMPlexSerif-Medium.283fca3d.ttf) format("truetype")}@font-face{font-family:"IBM Plex Serif";font-weight:700;font-style:regular;src:url(../../static/media/IBMPlexSerif-Bold.58c7f8d5.ttf) format("truetype")}@font-face{font-family:"IBM Plex Serif";font-weight:300;font-style:italic;src:url(../../static/media/IBMPlexSerif-LightItalic.1979fd74.ttf) format("truetype")}@font-face{font-family:"IBM Plex Serif";font-weight:400;font-style:italic;src:url(../../static/media/IBMPlexSerif-Italic.bc84e537.ttf) format("truetype")}@font-face{font-family:"IBM Plex Serif";font-weight:500;font-style:italic;src:url(../../static/media/IBMPlexSerif-MediumItalic.fe93e7da.ttf) format("truetype")}@font-face{font-family:"IBM Plex Serif";font-weight:700;font-style:italic;src:url(../../static/media/IBMPlexSerif-BoldItalic.346afa5a.ttf) format("truetype")}body{font-size:1rem}h1{font-size:calc(1.35rem + 1.2vw)}@media (min-width:1200px){h1{font-size:2.25rem}}h2{font-size:calc(1.3rem + .6vw)}@media (min-width:1200px){h2{font-size:1.75rem}}h3{font-size:1.25rem}h4{font-size:1rem}h5,h6{font-size:.8rem}small{font-size:.875em}sub,sup{font-size:.75em}code,kbd,pre{font-size:1em}pre{font-size:.875em}pre code{font-size:inherit}code,kbd{font-size:.875em}kbd{border-radius:.25rem}kbd kbd{font-size:1em}button,input,optgroup,select,textarea{font-size:inherit}[type=button]:not(:disabled),[type=reset]:not(:disabled),[type=submit]:not(:disabled),button:not(:disabled){cursor:pointer}legend{font-size:calc(1.275rem + .3vw)}@media (min-width:1200px){legend{font-size:1.5rem}}
/*# sourceMappingURL=main.e190763c.chunk.css.map */