(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[36],{4045:function(e,t,n){"use strict";n.r(t),n.d(t,"default",(function(){return c}));var a=n(11),r=n(13),i=n(25),o=n(26),s=n(0),l=n.n(s),u=n(225),p=n(5),c=function(e){Object(i.a)(n,e);var t=Object(o.a)(n);function n(){var e;Object(a.a)(this,n);for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];return(e=t.call.apply(t,[this].concat(i))).state={value:e.initialValue},e.setWidgetValue=function(t){e.props.widgetMgr.setIntValue(e.props.element,e.state.value,t)},e.onChange=function(t){e.setState({value:t},(function(){return e.setWidgetValue({fromUi:!0})}))},e.render=function(){var t=e.props.element,n=t.options,a=t.help,r=t.label,i=e.props.disabled;return Object(p.jsx)(u.b,{label:r,options:n,disabled:i,width:e.props.width,onChange:e.onChange,value:e.state.value,help:a})},e}return Object(r.a)(n,[{key:"initialValue",get:function(){var e=this.props.widgetMgr.getIntValue(this.props.element);return void 0!==e?e:this.props.element.default}},{key:"componentDidMount",value:function(){this.setWidgetValue({fromUi:!1})}}]),n}(l.a.PureComponent)}}]);
//# sourceMappingURL=36.114d10c3.chunk.js.map