/*! For license information please see 30.80ce15a9.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[30],{4039:function(e,t,r){"use strict";r.r(t),r.d(t,"default",(function(){return i}));var n=r(0),a=r(12),c=r(68),s=r(5);function i(e){var t=e.element,r=e.width,i=Object(n.useRef)(null),o=t.type,u=t.url;Object(n.useEffect)((function(){var e=i.current,r=function(){e&&(e.currentTime=t.startTime)};return e&&e.addEventListener("loadedmetadata",r),function(){e&&e.removeEventListener("loadedmetadata",r)}}),[t]);if(o===a.s.Type.YOUTUBE_IFRAME){var l=.75*r;return Object(s.jsx)("iframe",{title:u,src:function(e){var r=t.startTime;return r?"".concat(e,"?start=").concat(r):e}(u),width:r,height:l,frameBorder:"0",allow:"autoplay; encrypted-media",allowFullScreen:!0})}return Object(s.jsx)("video",{ref:i,controls:!0,src:Object(c.b)(u),className:"stVideo",style:{width:r}})}}}]);
//# sourceMappingURL=30.80ce15a9.chunk.js.map