(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[33],{4041:function(e,t,n){"use strict";n.r(t),n.d(t,"default",(function(){return c}));var a=n(11),r=n(13),i=n(25),l=n(26),o=n(0),s=n.n(o),u=n(158),p=n(5),c=function(e){Object(i.a)(n,e);var t=Object(l.a)(n);function n(){var e;Object(a.a)(this,n);for(var r=arguments.length,i=new Array(r),l=0;l<r;l++)i[l]=arguments[l];return(e=t.call.apply(t,[this].concat(i))).state={value:e.initialValue},e.setWidgetValue=function(t){e.props.widgetMgr.setStringValue(e.props.element,e.state.value,t)},e.onColorClose=function(t){e.setState({value:t},(function(){return e.setWidgetValue({fromUi:!0})}))},e.render=function(){var t=e.props,n=t.element,a=t.width,r=t.disabled,i=e.state.value;return Object(p.jsx)(u.a,{label:n.label,help:n.help,onChange:e.onColorClose,disabled:r,width:a,value:i})},e}return Object(r.a)(n,[{key:"initialValue",get:function(){var e=this.props.widgetMgr.getStringValue(this.props.element);return void 0!==e?e:this.props.element.default}},{key:"componentDidMount",value:function(){this.setWidgetValue({fromUi:!1})}}]),n}(s.a.PureComponent)}}]);
//# sourceMappingURL=33.72d7f62d.chunk.js.map